"""AC-API リクエストサンプル
"""

from urllib import parse, request
import json
import ac_authenticator


def sign_url(host, uri, access_key, private_key):
    # アクセスキーを接続する
    if '?' in uri:
        uri += ('&access_key=' + access_key)
    else:
        uri += ('?access_key=' + access_key)

    # アクセスキー付きのURIを署名
    authenticator = ac_authenticator.get_authenticator()
    signed_uri = authenticator.sign(uri, private_key)
    # 接続してURLを作成
    return host + uri + '&sign=' + signed_uri


def request_api(url):
    req = request.Request(url)
    with request.urlopen(req) as response:
        json_data = json.loads(response.read().decode('utf-8'))
    return json_data


if __name__ == "__main__":
    # インデクサーアドレス
    host = 'http://indexer.artistcommons.org'
    # アクセスキー(実際に登録されているアクセスキーを使用してください)
    access_key = 'sample'
    # 秘密鍵(実際に登録されている公開鍵に対応する秘密鍵を使用してください)
    with open('./id_rsa', 'r') as f:
        private_key = f.read()
    # APIリクエスト例
    name = parse.quote_plus('三浦大知')
    uri = '/api/v0/id/artist/search/acid?name=' + name

    # 署名付きURL生成
    signed_url = sign_url(host, uri, access_key, private_key)
    print("signed url:")
    print(signed_url)

    # APIリクエスト
    data = request_api(signed_url)
    print('response:')
    print(json.dumps(data, indent=2, ensure_ascii=False))
