"""AC-API 署名・検証サンプル
"""

from urllib import parse
import ac_authenticator


def sign_url(host, uri, access_key, private_key):
    """署名
    """
    # アクセスキーを接続する
    if '?' in uri:
        uri += ('&access_key=' + access_key)
    else:
        uri += ('?access_key=' + access_key)

    # アクセスキー付きのURIを署名
    authenticator = ac_authenticator.get_authenticator()
    signed_uri = authenticator.sign(uri, private_key)
    # 接続してURLを作成
    return host + uri + '&sign=' + signed_uri


def verify_url(url, public_key):
    """検証
    """
    # URIを抽出(スキームとドメインを取り除く)
    uri_pos = url.find('/api')
    uri = url[uri_pos:]
    # 署名とそれ以外を分離
    sign_pos = uri.find('&sign=')
    wo_sign = uri[:sign_pos]
    sign = uri[sign_pos + len('&sign='):]

    # 検証
    authenticator = ac_authenticator.get_authenticator()
    return authenticator.verify(wo_sign, sign, public_key)


if __name__ == "__main__":
    # 署名

    # プロバイダーアドレス
    host = 'https://example.com'
    # アクセスキー
    access_key = 'sample'
    # 秘密鍵
    with open('./id_rsa', 'r') as f:
        private_key = f.read()
    # APIリクエスト例
    name = parse.quote_plus('パフューム')
    uri = '/api/v0/id/artist/search/acid?name=' + name

    # 署名付きURL生成
    signed_url = sign_url(host, uri, access_key, private_key)
    print('signed url:')
    print(signed_url)


    # 検証

    # 公開鍵(実際には access_key から対応する公開鍵を見つける)
    with open('./id_rsa.pub', 'r') as f:
        public_key = f.read()

    # 署名付きURL検証
    result = verify_url(signed_url, public_key)
    print('verified result:')
    print(result)
