ArtistCommons API  Version 1.0.3

仕様書/シーケンス図

2021-03-19

# 目次


<!-- MarkdownTOC depth=4 -->

- 対象範囲
- 用語の定義
- ArtistCommonsサーバ
	- indexer, id converter, data provider　と、applicationの関係
- ArtistCommons API
	- API接続認証
	- クライアント登録のシーケンス
	- ID API
		- リクエストフォーマット
		- レスポンスフォーマット
	- ID変換のシーケンス
	- Data API
		- リクエストフォーマット
		- レスポンスフォーマット　（共通部分）
		- レスポンスフォーマット　（各データタイプ固有の部分）
			- NowOnAir
			- 楽曲
			- オーディオシーケンス
			- オーティオレコーディング
	- DATA取得のシーケンス
	- indexerによるキャッシュ
	- Index API
		- リクエストフォーマット
		- レスポンスフォーマット
	- INDEX取得のシーケンス


<!-- /MarkdownTOC -->


# 対象範囲

* データプロバイダ
  * NowOnAir情報サーバ
  * 楽曲情報サーバ
  * オーディオシーケンスサーバ
  * オーディオレコーディングサーバ

* 設計の対象
  * 通信仕様
  * 接続認証
  * プロトコル
  * リクエスト
  * レスポンス


# 用語の定義

* ArtistCommons サーバ：
  * ArtistCommonsはIPプロトコルで相互接続された複数のコンピューターで協調的に動作することを前提とする。動作しているそれぞれのコンピューターをArtistCommonsサーバーと呼ぶ。

* ArtistCommons API：
  * ArtistCommonsサーバー間の協調動作は、IPプロトコル通信で行われる。その際のリクエスト及びレスポンスの動作仕様や書式を決めたものをArtistCommons API(Application Programming Interfaceの略）と呼ぶ。
  * ArtistCommons APIで扱う情報は、IDとDATAに大別される。

* ID :
  * DATAの識別子。
  　具体的には、ソフトエウア処理のためのユニークな識別子と、人間が読むための名前のペアで構成される。
  　ID体系は複数を許容し、それらの関連付け（ID変換）を可能とする。

* DATA :
  * コンテンツやそのメタデータ。コンテンツには、その識別子としてIDを付与する。
  * メタデータの体系は、世界的なメタデータスキーマ共通化プロジェクト　http://schema.org/ に規定されたproperty、及びそのJSON-LD書式に準拠し、必要に応じて項目を追加・拡張する。

* API接続認証 :
  * ArtistCommons APIの接続時に、接続元は、シークレットキーを用いてリクエストの一部にデジタル署名を行い、それをアクセスキーに加えて接続先に通知する。この動作を、ArtistCommonsにおける署名によるAPI接続認証と呼ぶ。
  * アクセスキーとシークレットキーは、API利用登録時に接続先から払い出されるものとする。



# ArtistCommonsサーバ

indexer, id converter, data provider の3種類のサーバを定義する。

1サーバで複数機能を兼ねてもよい。

* indexer
  * アプリケーションへのレスポンス
  * 他のインデクサの処理可能リストの管理
  * レスポンスキャッシュの管理
  * プロバイダへのリクエスト

* id converter
  * インデクサへのレスポンス
  * ID変換テーブルの管理

* data provider
  * インデクサへのレスポンス
  * コンテンツデータの管理

ArtistCommonsを利用するシステム（アプリケーションサーバ等）を、applicationと記載する

## indexer, id converter, data provider　と、applicationの関係
![ArtistCommonsサーバ](img/rapf-server.jpeg)

* indexerは、配下のapplication, id converter, data providerのやり取りを中継するほか、
他のindexerと連携する。



# ArtistCommons API

## API接続認証

以下をリクエストクエリに追加する。

```
?access_key={string}&sign={string}
```

access_key: サーバ管理者から払い出されたアクセスキー

sign: サーバ管理者から払い出されたシークレットキーでAPIリクエストを署名したもの

* クライアント手順
 * 1. サーバへの登録時にアクセスキー、シークレットキーを取得しておく(手動)
 *  2. リクエスト時、ドメイン、signクエリを除いたリクエスト文字列からSHA256を使ってダイジェスト値を取得
 *  3. ダイジェスト値をRSAシークレットキーで署名
 *  4. 署名した値をbase64で符号化し、"+"を"-"に、"/"を"_"に置き換える
 *  5. 上記符号を{sign}に付加してリクエストする

* サーバ手順
 * 1. クライアント登録時にアクセスキー、シークレットキーを払い出し、アクセスキー、パブリックキーを保存しておく
 * 2. レスポンス時、signクエリの"-"を"+"に、"_"を"/"に置き換え、base64で復号化し、署名の値を取得
 * 3. ドメイン、signクエリを除いたリクエスト文字列からSHA256を使ってダイジェスト値を取得
 * 4. RSAパブリックキーを使い、ダイジェスト値と署名の値を検証
 * 5. 検証成功ならレスポンス処理続行

## クライアント登録のシーケンス
![クライアント登録](img/client-registration.jpeg)

* operatorは管理画面等による人間の操作を表す。

* clientはリクエスト元を、serverはリクエスト先を表す。

* この図でのID登録は、ArtistCommons利用におけるclientとしてのID。（コンテンツのIDではない）

* 右下の記載の意味：
このシーケンスが該当する組み合わせは
  * clientがapplicationで、serverがindexerの場合
  * clientがindexerで、serverがproviderの場合
  * clientがindexerで、serverがindexerの場合



## ID API

あるIDタイプのIDを別のIDタイプのIDに変換する

id converter, indexer が提供する

### リクエストフォーマット

```
GET /api/v0/id/{data_type}/{id_type}/{id}
GET /api/v0/id/{data_type}/{id_type}/{id}/{to_id_type}
```
```
?provider={string}
```

* data_type: データタイプ(楽曲、音楽家、等)
* id_type: IDタイプ(ISRC、アーティストコモンズID、等)
* id: IDタイプによるID
* to_id_type: 変換先IDタイプ
* provider: プロバイダ指定

ID検索

```
GET /api/v0/id/{data_type}/search
GET /api/v0/id/{data_type}/search/{to_id_type}
```
```
?name={string}&provider={string}
```

* name: 名前(楽曲名、アーティスト名、等)
* provider: プロバイダ指定

各検索条件クエリは省略可能

### レスポンスフォーマット

ステータスコード

* 200: 成功
* 400: パラメータエラー
* 401: 認証エラー
* 404: リソースが存在しない
* 500: その他サーバエラー

成功時

```
{
  "status": "succeeded",
  "request": {
    "uri": "{request_uri}"
  },
  "data": [
    {
      "indexer": "{indexer_url}",
      "provider": "{provider_url}",
      "published_at": "{published_datetime}",
      "expired_at": "{expired_datetime}",
      "body": {
        "@context": "http://schema.org",
        "@type": "Person",
        "identifier": {
          "@type": "PropertyValue",
          "propertyID": "{id_type}",
          "value":  "{id}"
        },
        "name": "{performer_name}"
      }
    }
  ]
}
```

失敗時

```
{
  "status": "failed",
  "request": {
    "uri": "{request_uri}"
  },
  "error": {
    "message": "{error_message}"
  }
}
```

日時の書式はISO 8601
```
2018-06-28T10:32:21+09:00
```

## ID変換のシーケンス
![ID変換](img/id-data-request.jpeg)

* あるID体系のIDまたはnameでリクエストすると、希望するID体系のIDがレスポンスされる。

* シーケンス動作は、IDとDATAで同じ。

## Data API

IDから詳細データを返却する
data provider, indexer が提供する

### リクエストフォーマット

```
GET /api/v0/data/{data_type}/{id_type}/{id}
```
```
?begun_at={datetime}&ended_at={datetime}&provider={string}
```

* data_type: データタイプ(楽曲、音楽家、等)
* id_type: IDタイプ(ISRC、アーティストコモンズID、等)
* id: IDタイプによるID
* begun_at: 検索期間開始日時(チケットのイベント日時、等)
* ended_at: 検索期間終了日時
* provider: プロバイダ指定

日時の書式はISO 8601をURLエンコードする
```
2018-06-28T10%3A32%3A21%2B09%3A00
```

### レスポンスフォーマット　（共通部分）

ステータスコード

* 200: 成功
* 400: パラメータエラー
* 401: 認証エラー
* 404: リソースが存在しない
* 500: その他サーバエラー

成功時
{content}には以下の項の情報が入る

```
{
  "status": "succeeded",
  "request": {
    "uri": "{request_uri}"
  },
  "data": [
    {
      "indexer": "{indexer_url}",
      "provider": "{provider_url}",
      "published_at": "{published_datetime}",
      "expired_at": "{expired_datetime}",
      "body": "{content}"
    }
  ]
}
```

失敗時

```
{
  "status": "failed",
  "request": {
    "uri": "{request_uri}"
  },
  "error": {
    "message": "{error_message}"
  }
}
```

日時の書式はISO 8601
```
2018-06-28T10:32:21+09:00
```

### レスポンスフォーマット　（各データタイプ固有の部分）


#### NowOnAir

Type: http://schema.org/PlayAction

* identifier (http://schema.org/PropertyValue): ID
* hasPart(http://schema.org/PlayAction): 子再生アクション
  "hasPart" はカスタム拡張プロパティ
  * object (http://schema.org/MusicRecording): 対象楽曲
  * startTime (http://schema.org/DateTime): 開始日時
  * endTime (http://schema.org/DateTime): 終了日時

```
{
  "@context": "http://schema.org",
  "@type": "PlayAction",
  "identifier": {
    "@type": "PropertyValue",
    "propertyID": "{id_type}",
    "value":  "{id}"
  },
  "hasPart": [
    {
      "@type": "PlayAction",
      "object": {
        "@type": "MusicRecording",
        "identifier": {
          "@type": "PropertyValue",
          "propertyID": "{id_type}",
          "value":  "{id}"
        },
        "name": "{song_name}",
        "byArtist": {
          "@type": "MusicGroup",
          "identifier": {
            "@type": "PropertyValue",
            "propertyID": "{id_type}",
            "value":  "{id}"
          },
          "name": "{artist_name}"
        }
      },
      "startTime": "{start_datetime}",
      "endTime": "{end_datetime}"
    }
  ]
}
```


#### 楽曲

Type: http://schema.org/MusicRecording

* identifier (http://schema.org/PropertyValue): ID
* name (string): 名前
* byArtist (http://schema.org/Organization|http://schema.org/Person): アーティスト
* inAlbum (http://schema.org/MusicAlbum): アルバム
* audio (http://schema.org/AudioObject): 音源
* image (http://schema.org/ImageObject): 楽曲画像
* contributor (http://schema.org/Organization|http://schema.org/Person): 貢献者

```
{
  "@context": "http://schema.org",
  "@type": "MusicRecording",
  "identifier": {
    "@type": "PropertyValue",
    "propertyID": "{id_type}",
    "value":  "{id}"
  },
  "name": "{song_name}",
  "byArtist": {
    "@type": "MusicGroup",
    "identifier": {
      "@type": "PropertyValue",
      "propertyID": "{id_type}",
      "value":  "{id}"
    },
    "name": "{artist_name}",
  },
  "inAlbum": {
    "@type": "MusicAlbum",
    "identifier": {
      "@type": "PropertyValue",
      "propertyID": "{id_type}",
      "value":  "{id}"
    },
    "name": "{album_name}"
  },
  "audio": {
    "@type": "AudioObject",
    "contentUrl": "{audio_url}"
  },
  "image": {
    "@type": "ImageObject",
    "contentUrl": "{image_url}"
  },
  "contributor": [
    {
      "@type": "Person",
      "identifier": {
        "@type": "PropertyValue",
        "propertyID": "{id_type}",
        "value":  "{id}"
      },
      "name": "{artist_name}",
      "jobTitle": "{role}"
    }
  ]
}
```


#### オーディオシーケンス

Type: AudioSequence
  "AudioSequence" は "http://schema.org/CreativeWork"のカスタム派生タイプ

* identifier (http://schema.org/PropertyValue): ID
* name (string): 名前
* description (string): 概要
* hasPart (AudioSequence|AudioRecording): 子オーディオシーケンス・オーディオレコーディング
  * startDuration (http://schema.org/Duration): 子オーディオシーケンスの相対開始時間
    "startDuration" はカスタム拡張プロパティ
  * endDuration (http://schema.org/Duration): 子オーディオシーケンスの相対終了時間
    "endDuration" はカスタム拡張プロパティ

```
{
  "@type": "AudioSequence",
  "identifier": {
    "@type": "PropertyValue",
    "propertyID": "{id_type}",
    "value":  "{id}"
  },
  "name": "{audio_sequence_name}",
  "description": "{audio_sequence_description}",
  "hasPart": [
    {
      "@type": "AudioSequence",
      "identifier": {
        "@type": "PropertyValue",
        "propertyID": "{id_type}",
        "value":  "{id}"
      },
      "name": "{audio_sequence_name}",
      "startDuration": "{start_duration}",
      "endDuration": "{end_duration}"
    }
  ]
}
```




#### オーティオレコーディング

Type: AudioRecording
  "AudioRecording" は "http://schema.org/CreativeWork"のカスタム派生タイプ

* identifier (http://schema.org/PropertyValue): ID
* name (string): 名前
* audio (http://schema.org/AudioObject): 音源
* duration (http://schema.org/Duration): 再生時間
* contributor (http://schema.org/Organization|http://schema.org/Person): 貢献者

```
{
  "@context": "http://schema.org",
  "@type": "AudioObject",
  "identifier": {
    "@type": "PropertyValue",
    "propertyID": "{id_type}",
    "value":  "{id}"
  },
  "name": "{audio_name}",
  "audio": {
    "@type": "AudioObject",
    "contentUrl": "{audio_url}"
  },
  "duration": "{audio_duration}",
  "contributor": [
    {
      "@type": "Person",
      "identifier": {
        "@type": "PropertyValue",
        "propertyID": "{id_type}",
        "value":  "{id}"
      },
      "name": "{performer_name}"
    }
  ]
}
```



## DATA取得のシーケンス
![DATA取得シーケンス](img/id-data-request.jpeg)

* 欲しいDATA（コンテンツ及びメタデータ）の、datatypeと、ID体系とIDのセットを指定してリクエストすると、希望するDATAがレスポンスされる。

* シーケンス動作は、IDとDATAで同じ。

## indexerによるキャッシュ
![indexerキャッシュ](img/indexer-cache.jpeg)

* indexerは、IDやDATAをキャッシュする。その際のキャッシュ時間は、expired_atプロパティーに従う。
ID/DATAリクエストに対しては、まずindexer自身のキャッシュを参照し、存在しない場合のみ該当するプロバイダにリクエストする。


## Index API

どんなデータが処理可能かのリストを返却する
id converter, data provider, indexer が提供する

### リクエストフォーマット

```
GET /api/v0/index
```

### レスポンスフォーマット

ステータスコード

* 200: 成功
* 401: 認証エラー
* 500: その他サーバエラー

成功時

```
{
  "status": "succeeded",
  "request": {
    "uri": "{request_uri}"
  },
  "data": [
    {
      "indexer": "{indexer_url}",
      "provider": "{provider_url}"
      "provider_type": "{provider_type}",
      "data_type": "{data_type}",
      "id_type": "{id_type}",
      "to_id_types": [
        {id_type}
      ],
      "search_keys": [
        {search_key}
      ]
    }
  ]
}
```

失敗時

```
{
  "status": "failed",
  "request": {
    "uri": "{request_uri}"
  },
  "error": {
    "message": "{error_message}"
  }
}
```


## INDEX取得のシーケンス
![INDEX取得シーケンス](img/index-request.jpeg)

* IDやDATAについて、どのプロバイダがどのような検索に対応しているかを取得するシーケンス。

* indexerは、IDプロバイダやDATAプロバイダに問い合わせて、どのような情報があるかを把握する他、連携する他のindexerに定期的に問い合わせを行い、結果を保持することにより、自分の配下のプロバイダに情報が無い場合に他のindexerへ問い合わせることを可能にする。
