# AC-API 追加データフォーマット

AC Data API の追加データフォーマット仕様。(ラジオ番組、ニュース、物販）

Data Provider に必要な Index API 仕様。

<!-- TOC -->

- [AC-API 追加データフォーマット](#ac-api-追加データフォーマット)
    - [全API共通仕様](#全api共通仕様)
        - [API接続認証](#api接続認証)
    - [Data API 仕様](#data-api-仕様)
        - [共通仕様](#共通仕様)
            - [リクエストフォーマット](#リクエストフォーマット)
            - [レスポンスフォーマット](#レスポンスフォーマット)
        - [ラジオ番組](#ラジオ番組)
            - [リクエスト](#リクエスト)
            - [レスポンス](#レスポンス)
        - [ニュース](#ニュース)
            - [リクエスト](#リクエスト-1)
            - [レスポンス](#レスポンス-1)
        - [物販](#物販)
            - [リクエスト](#リクエスト-2)
            - [レスポンス](#レスポンス-2)
    - [Index API 仕様](#index-api-仕様)
        - [共通仕様](#共通仕様-1)
            - [リクエストフォーマット](#リクエストフォーマット-1)
            - [レスポンスフォーマット](#レスポンスフォーマット-1)
        - [ラジオ番組](#ラジオ番組-1)
            - [レスポンス](#レスポンス-3)
        - [ニュース](#ニュース-1)
            - [レスポンス](#レスポンス-4)
        - [物販](#物販-1)
            - [レスポンス](#レスポンス-5)
    - [(付録)追加語彙](#付録追加語彙)
        - [リクエストデータタイプ](#リクエストデータタイプ)
        - [リクエストクエリ](#リクエストクエリ)
        - [レスポンスIDタイプ](#レスポンスidタイプ)

<!-- /TOC -->

## 全API共通仕様

前提としての共通仕様の要約。
詳細は別紙。

### API接続認証

以下をリクエストクエリに追加する。

```
?access_key={string}&sign={string}
```

access_key: サーバ管理者から払い出されたアクセスキー
sign: サーバ管理者から払い出されたシークレットキーでAPIリクエストを署名したもの

## Data API 仕様

### 共通仕様

#### リクエストフォーマット

```
GET /api/v0/data/{data_type}/{id_type}/{id}
```

* data_type: データタイプ(楽曲、音楽家、等)
* id_type: IDタイプ(ISRC、アーティストコモンズID、等)
* id: IDタイプによるID

日時の書式はISO 8601をURLエンコードする
```
2018-06-28T10%3A32%3A21%2B09%3A00
```

ex.
```
https://example.com/api/v0/data/artist/acid/ACJP-AZZ-000-F0001-6
```

#### レスポンスフォーマット

ステータスコード

* 200: 成功
* 400: パラメータエラー
* 401: 認証エラー
* 404: リソースが存在しない
* 500: その他サーバエラー

成功時
{content} には以下の項(ラジオ番組、ニュース、物販)の情報が入る

```json
{
  "status": "succeeded",
  "request": {
    "uri": "{request_uri}"
  },
  "data": [
    {
      "indexer": "{indexer_url}",
      "provider": "{provider_url}",
      "published_at": "{published_datetime}",
      "expired_at": "{expired_datetime}",
      "body": "{content}"
    }
  ]
}
```
* request/uri: URLからプロトコル、ドメイン、access_key、signを除いたURI
    ex. /api/v0/id/artist/search/acid?name=%E7%B1%B3%E6%B4%A5%E7%8E%84%E5%B8%AB
* data[]/indexer: IndexerのURL、自身が Data Provider の場合は null
* data[]/provider: Data ProviderのURL、プロトコルとドメイン
    ex. https://acid.artistcommons.org
* data[]/published_at: 情報取得日時(ISO8601)
* data[]/expired_at: 情報が無効になる日時(キャッシュされた場合に無効にしたい日時)(ISO8601)
* data[]/body: 実際の情報

失敗時

```json
{
  "status": "failed",
  "request": {
    "uri": "{request_uri}"
  },
  "error": {
    "message": "{error_message}"
  }
}
```

日時の書式はISO 8601
```
2018-06-28T10:32:21+09:00
```

### ラジオ番組

入力はアーティストID(acid)、出力はラジオ番組情報

#### リクエスト

```
GET /api/v0/data/radio_event/acid/{acid}
```
```
?begun_at={datetime}&ended_at={datetime}&order={order}
```

* {acid}: ACID
* begun_at(optional): 検索期間開始日時(ISO8601をURLエンコード)
* ended_at(optional): 検索期間終了日時(ISO8601をURLエンコード)
* order(optional): 放送開始日時のソート順
    asc: 昇順, desc: 降順

ex.
```
https://example.com/api/v0/data/radio_event/acid/ACJP-AZZ-000-F0001-6?begun_at=2021-01-13T00%3A00%3A00%2B09%3A00&ended_at=2021-01-14T00%3A00%3A00%2B09%3A00&order=asc
```

#### レスポンス

共通仕様の {content} の内容

Type: RadioBroadcastEvent
    https://schema.org/BroadcastEvent の独自派生

```json
{
    "@context": "http://schema.org",
    "@type": "RadioBroadcastEvent",
    "identifier": {
        "@type": "PropertyValue",
        "propertyID": "radio_event_radiko",
        "value": "{id}"
    },
    "name": "{event_name}",
    "publishedOn": {
        "@type": "BroadcastService",
        "identifier": {
            "@type": "PropertyValue",
            "propertyID": "channel_radiko",
            "value":  "{id}"
        },
        "name": "{channel_name}"
    },
    "startDate": "{datetime}",
    "endDate": "{datetime}",
    "image": {
        "@type": "ImageObject",
        "contentUrl": "{image_url}"
    },
    "mainEntityOfPage": "{event_url}",
    "url": "{radiko_url}",
    "about": {
        "@type": "RadioEpisode",
        "identifier": {
            "@type": "PropertyValue",
            "propertyID": "radio_episode_radiko",
            "value":  "{id}"
        },
        "name": "{episode_name}",
        "partOfSeason": {
            "@type": "RadioSeason",
            "identifier": {
                "@type": "PropertyValue",
                "propertyID": "radio_season_radiko",
                "value":  "{id}"
            },
            "name": "{season_name}",
        },
        "partOfSeries": {
            "@type": "RadioSeries",
            "identifier": {
                "@type": "PropertyValue",
                "propertyID": "radio_series_radiko",
                "value":  "{id}"
            }
        }
    }
}
```

* identifier/value: イベントID
* name: イベント名
* publishedOn/identifier/value: ラジオ局ID
* publishedOn/name: ラジオ局名
* startDate: 放送開始日時(ISO8601)
* endDate: 放送終了日時(ISO8601)
* image/contentUrl: イベントサムネイル画像
* mainEntityOfPage: 番組HPのURL
* url: radikoの再生URL
* about/identifier/value: エピソードID
* about/name: エピソード名(実装メモ、ないのでイベント名を使用する)
* about/partOfSeason/identifier/value: シーズンID
* about/partOfSeason/name: シーズン名
* about/partOfSeries/identifier/value: シリーズID

### ニュース

入力はアーティストID(acid)、出力はニュース情報

#### リクエスト

```
GET /api/v0/data/news/acid/{acid}
```
```
?begun_at={datetime}&ended_at={datetime}&order={order}
```

* {acid}: ACID
* begun_at(optional): 検索期間開始日時(ISO8601をURLエンコード)
* ended_at(optional): 検索期間終了日時(ISO8601をURLエンコード)
* order(optional): 公開日時のソート順
    asc: 昇順, desc: 降順

ex.
```
https://example.com/api/v0/data/news/acid/ACJP-AZZ-000-F0001-6?begun_at=2021-01-13T00%3A00%3A00%2B09%3A00&ended_at=2021-01-14T00%3A00%3A00%2B09%3A00&order=asc
```

#### レスポンス

共通仕様の {content} の内容

Type: https://schema.org/NewsArticle

```json
{
    "@context": "http://schema.org",
    "@type": "NewsArticle",
    "identifier": {
        "@type": "PropertyValue",
        "propertyID": "news_barks",
        "value": "{id}"
    },
    "publisher": {
        "@type": "Organization",
        "identifier": {
            "@type": "PropertyValue",
            "propertyID": "news_publisher_ac",
            "value": "barks"
        },
        "name": "BARKS",
        "url": "https://www.barks.jp"
    },
    "headline": "{headline}",
    "articleBody": "{article_body}",
    "url": "{news_url}",
    "datePublished": "{datetime}",
    "image": [
        {
            "@type": "ImageObject",
            "contentUrl": "{image_url}"
        },
    ]
}
```

* identifier/value: 記事ID
* headline: 記事タイトル
* articleBody: 記事内容
* url: 記事URL
* datePublished: 公開日時(ISO8601)
* image[]/contentUrl: サムネイル画像URL

ex.
```json
{
    "status": "succeeded",
    "request": {
        "uri": "/api/v0/data/news/acid/ACJP-AZZ-000-F0001-6?begun_at=2021-01-13T00%3A00%3A00%2B09%3A00&ended_at=2021-01-14T00%3A00%3A00%2B09%3A00&order=asc"
    },
    "data": [
        {
            "indexer": null,
            "provider": "https://example.com",
            "published_at": "2011-01-14T00:00:00+09:00",
            "expired_at": "2011-01-14T01:00:00+09:00",
            "body": {
                "@context": "http://schema.org",
                "@type": "NewsArticle",
                "identifier": {
                    "@type": "PropertyValue",
                    "propertyID": "news_barks",
                    "value": "123456"
                },
                "publisher": {
                    "@type": "Organization",
                    "identifier": {
                        "@type": "PropertyValue",
                        "propertyID": "news_publisher_ac",
                        "value": "barks"
                    },
                    "name": "BARKS",
                    "url": "https://www.barks.jp"
                },
                "headline": "記事タイトル1",
                "articleBody": "記事概要1",
                "url": "https://www.barks.jp/news/?id=123456",
                "datePublished": "2011-01-13T00:00:00+09:00",
                "image": [
                    {
                        "@type": "ImageObject",
                        "contentUrl": "https://img.barks.jp/image/review/123456/0.jpg"
                    },
                    {
                        "@type": "ImageObject",
                        "contentUrl": "https://img.barks.jp/image/review/123456/1.jpg"
                    }
                ]
            }
        },
        {
            "indexer": null,
            "provider": "https://example.com",
            "published_at": "2011-01-14T00:00:00+09:00",
            "expired_at": "2011-01-14T01:00:00+09:00",
            "body": {
                "@context": "http://schema.org",
                "@type": "NewsArticle",
                "identifier": {
                    "@type": "PropertyValue",
                    "propertyID": "news_barks",
                    "value": "123457"
                },
                "publisher": {
                    "@type": "Organization",
                    "identifier": {
                        "@type": "PropertyValue",
                        "propertyID": "news_publisher_ac",
                        "value": "barks"
                    },
                    "name": "BARKS",
                    "url": "https://www.barks.jp"
                },
                "headline": "記事タイトル2",
                "articleBody": "記事概要2",
                "url": "https://www.barks.jp/news/?id=123457",
                "datePublished": "2011-01-13T01:00:00+09:00",
                "image": [
                    {
                        "@type": "ImageObject",
                        "contentUrl": "https://img.barks.jp/image/review/123457/0.jpg"
                    },
                    {
                        "@type": "ImageObject",
                        "contentUrl": "https://img.barks.jp/image/review/123457/1.jpg"
                    }
                ]
            }
        }
    ]
}
```

### 物販

入力はアーティストID(acid)、出力は物販情報

#### リクエスト

```
GET /api/v0/data/product/acid/{acid}
```
```
?status={status}&per_page={per_page}&page={page}
```

* {acid}: ACID
* status(optional): 商品ステータス
    new: NEW, small_stock: 残りわずか, bonus_point: ポイントアップ中, recommended: オススメ, out_of_stock: 在庫切れ
* per_page(optional): 1ページあたりの取得件数、指定された場合はページネーションが有効になる、デフォルト値は適当に定める
* page(optional): 取得するページ番号、指定された場合はページネーションが有効になる、開始値は1から、デフォルト値は1

(ソート方法はこの仕様では定めない)

ex.
```
https://example.com/api/v0/data/product/acid/ACJP-AZZ-000-F0001-6?status=new&per_page=10&page=1
```

#### レスポンス

共通仕様の {content} の内容

Type: https://schema.org/Product

```json
{
    "@context": "http://schema.org",
    "@type": "Product",
    "identifier": {
        "@type": "PropertyValue",
        "propertyID": "product_fanplus",
        "value": "{id}"
    },
    "name": "{product_name}",
    "brand": {
        "@type": "PerformingGroup",
        "identifier": {
            "@type": "PropertyValue",
            "propertyID": "acid",
            "value": "{id}"
        },
        "name": "{artist_name}"
    },
    "color": "{color}",
    "offers": {
        "@type": "Offer",
        "price": "{price}",
        "priceCurrency": "JPY"
    },
    "url": "{product_url}",
    "image": [
        {
            "@type": "ImageObject",
            "contentUrl": "{image_url}"
        },
    ],
    "description": "{description}"
}
```

* identifier/value: プロダクトID
* name: プロダクト名
* brand/identifier/value: アーティストID(acid)
* brand/name: アーティスト名
* color: 色
* offers/price: 価格(日本円)
* url: 商品ページURL
* image[]/contentUrl: 画像URL
* description: 商品概要

ex.
```json
{
    "status": "succeeded",
    "request": {
        "uri": "/api/v0/data/product/acid/ACJP-AZZ-000-F0001-6?status=recommended&per_page=10&page=1"
    },
    "data": [
        {
            "indexer": null,
            "provider": "https://example.com",
            "published_at": "2011-01-14T00:00:00+09:00",
            "expired_at": "2011-01-14T01:00:00+09:00",
            "body": {
                "@context": "http://schema.org",
                "@type": "Product",
                "identifier": {
                    "@type": "PropertyValue",
                    "propertyID": "product_fanplus",
                    "value": "xxxxx-xxxx-xxxxx"
                },
                "name": "商品名1",
                "brand": {
                    "@type": "PerformingGroup",
                    "identifier": {
                        "@type": "PropertyValue",
                        "propertyID": "acid",
                        "value": "ACJP-AZZ-000-F0001-6"
                    },
                    "name": "ベルリン・フィルハーモニー管弦楽団"
                },
                "color": "ピンク",
                "offers": {
                    "@type": "Offer",
                    "price": "2800",
                    "priceCurrency": "JPY"
                },
                "url": "https://xxx.merchview.co.jp/xxx/xxxxx",
                "image": [
                    {
                        "@type": "ImageObject",
                        "contentUrl": "https://store.xxxx.jp/xxxxx/img1.png"
                    },
                    {
                        "@type": "ImageObject",
                        "contentUrl": "https://store.xxxx.jp/xxxxx/img2.png"
                    }
                ],
                "description": "説明文章1"
            }
        },
        {
            "indexer": null,
            "provider": "https://example.com",
            "published_at": "2011-01-14T00:00:00+09:00",
            "expired_at": "2011-01-14T01:00:00+09:00",
            "body": {
                "@context": "http://schema.org",
                "@type": "Product",
                "identifier": {
                    "@type": "PropertyValue",
                    "propertyID": "product_fanplus",
                    "value": "xxxxx-xxxx-xxxxy"
                },
                "name": "商品名2",
                "brand": {
                    "@type": "PerformingGroup",
                    "identifier": {
                        "@type": "PropertyValue",
                        "propertyID": "acid",
                        "value": "ACJP-AZZ-000-F0001-6"
                    },
                    "name": "ベルリン・フィルハーモニー管弦楽団"
                },
                "color": "白",
                "offers": {
                    "@type": "Offer",
                    "price": "2800",
                    "priceCurrency": "JPY"
                },
                "url": "https://xxx.merchview.co.jp/xxx/xxxxy",
                "image": [
                    {
                        "@type": "ImageObject",
                        "contentUrl": "https://store.xxxx.jp/xxxxy/img1.png"
                    },
                    {
                        "@type": "ImageObject",
                        "contentUrl": "https://store.xxxx.jp/xxxxy/img2.png"
                    }
                ],
                "description": "説明文章2"
            }
        }
    ]
}
```

## Index API 仕様

どんなデータが処理可能かのリストを返却する。
AC-API対応するためには必ず実装する必要があります。

### 共通仕様

#### リクエストフォーマット

```
GET /api/v0/index
```

ex.
```
https://example.com/api/v0/index
```

#### レスポンスフォーマット

ステータスコード

* 200: 成功
* 401: 認証エラー
* 500: その他サーバエラー

成功時

```json
{
  "status": "succeeded",
  "request": {
    "uri": "{request_uri}"
  },
  "data": [
    {
      "indexer": "{indexer_url}",
      "provider": "{provider_url}",
      "provider_type": "{provider_type}",
      "data_type": "{data_type}",
      "id_type": "{id_type}",
      "to_id_types": [
        "{id_type}"
      ],
      "search_keys": [
        "{search_key}"
      ]
    }
  ]
}
```
* request/uri: URLからプロトコル、ドメイン、access_key、signを除いたURI
    ex. /api/v0/index
* data[]/indexer: IndexerのURL、自身が Data Provider の場合は null
* data[]/provider: Data ProviderのURL、プロトコルとドメイン
    ex. https://acid.artistcommons.org
* data[]/provider_type: プロバイダタイプ
    id_converter: ID Converter, data_provider: Data Provider
* data[]/data_type: リクエストで使用できるデータタイプ
* data[]/id_type: リクエストで使用できるIDタイプ
* data[]/to_id_types: ID Converter の場合のみ変換先のIDタイプを列挙する
    Data Provider の場合は null
* data[]/search_keys: 使用可能なクエリパラメータ名を列挙する(access_key, sign は含まない)

失敗時

```json
{
  "status": "failed",
  "request": {
    "uri": "{request_uri}"
  },
  "error": {
    "message": "{error_message}"
  }
}
```

### ラジオ番組

#### レスポンス

```json
{
  "status": "succeeded",
  "request": {
    "uri": "/api/v0/index"
  },
  "data": [
    {
      "indexer": null,
      "provider": "{provider_url}",
      "provider_type": "data_provider",
      "data_type": "radio_event",
      "id_type": "acid",
      "to_id_types": null,
      "search_keys": [
          "begun_at", "ended_at", "order"
      ]
    }
  ]
}
```

### ニュース

#### レスポンス

```json
{
  "status": "succeeded",
  "request": {
    "uri": "/api/v0/index"
  },
  "data": [
    {
      "indexer": null,
      "provider": "{provider_url}",
      "provider_type": "data_provider",
      "data_type": "news",
      "id_type": "acid",
      "to_id_types": null,
      "search_keys": [
          "begun_at", "ended_at", "order"
      ]
    }
  ]
}
```

### 物販

#### レスポンス

```json
{
  "status": "succeeded",
  "request": {
    "uri": "/api/v0/index"
  },
  "data": [
    {
      "indexer": null,
      "provider": "{provider_url}",
      "provider_type": "data_provider",
      "data_type": "product",
      "id_type": "acid",
      "to_id_types": null,
      "search_keys": [
          "status", "per_page", "page"
      ]
    }
  ]
}
```

## (付録)追加語彙

### リクエストデータタイプ

* radio_event: ラジオ番組
* news: ニュース
* product: 物販

### リクエストクエリ

* begun_at: 検索期間開始日時(既存)
* ended_at: 検索期間終了日時(既存)
* order: ソート順 [asc, desc]
* status: 商品ステータス [new, small_stock, bonus_point, recommended, out_of_stock]
* per_page: 1ページあたりの取得件数
* page: 取得ページ番号

### レスポンスIDタイプ

* radio_event_radiko: ラジオ放送枠ID
* channel_radiko: ラジオ局ID(既存)
* radio_episode_radiko: ラジオエピソードID
* radio_season_radiko: ラジオシーズンID
* radio_series_radiko: ラジオシリーズID
* news_barks: 記事ID
* news_publisher_ac: ニュースパブリッシャID
* product_fanplus: 商品ID
